/* Filename - findinstr.c
 * 
 *  Overview: This example uses the FindLstn function to detect
 *  GPIB Listeners (devices) on the GPIB.  The GPIB board number
 *  is hardcoded below and may need to be changed if you have more
 *  than one GPIB board in your system.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ni4882.h"

#define GPIB0        0                 // Board index

static int Num_Instruments,            // Number of instruments on GPIB
           PAD,                        // Primary address
           SAD,                        // Secondary address
           loop;                       // Loop counter
static Addr4882_t Instruments[32],     // Array of primary addresses
                  Result[31];          // Array of listen addresses

static void GpibError(char *msg);


int main(void)  {

/*
 *  Your board needs to be the Controller-In-Charge in order to find
 *  all instrument on the GPIB.  To accomplish this, the function
 *  SendIFC is called.  If the error bit ERR is set in ibsta, call
 *  GpibError with an error message.
 */

    SendIFC(GPIB0);
    if (Ibsta() & ERR)
    {
       GpibError("SendIFC error");
       return 1;
    }

/*
 *  Create an array containing all valid GPIB primary addresses,
 *  except for primary address 0.  Your GPIB interface board is at
 *  address 0 by default.  This array (Instruments) will be given to
 *  the function FindLstn to find all instruments.  The constant
 *  NOADDR, defined in NI488.H, signifies the end of the array.
 */

    for (loop = 0; loop < 30; loop++) {
       Instruments[loop] = (Addr4882_t)(loop + 1);
    }
    Instruments[30] = NOADDR;

/*
 *  Print message to tell user that the program is searching for all
 *  active listeners.  Find all of the instruments on the bus.  Store
 *  the instrument addresses in the array Result. Note, the
 *  instruments must be powered on and connected with a GPIB cable in
 *  order for FindLstn to detect them. If the error bit ERR is set in
 *  ibsta, call GpibError with an error message.
 */

    printf("Finding all instruments on the bus...\n\n");

    FindLstn(GPIB0, Instruments, Result, 31);
    if (Ibsta() & ERR)
    {
       GpibError("FindLstn error");
       return 1;
    }

/*
 *  Ibcnt() returns the actual number of addresses stored in the
 *  Result array. Assign the value of Ibcnt() to the variable
 *  Num_Instruments. Print the number of instruments found.
 */

    Num_Instruments = Ibcnt();

    printf("Number of instruments found = %d\n\n", Num_Instruments);

/*
 *  The Result array contains the addresses of all the instruments
 *  found by FindLstn. Use the constant NOADDR, as defined in
 *  NI488.H, to signify the end of the array.
 */

    Result[Num_Instruments] = NOADDR;

/*
 *  Print out each instrument's PAD and SAD, one at a time.
 *
 *  Establish a FOR loop to print out the information. The variable
 *  LOOP will serve as a counter for the FOR loop and as the index
 *  to the array Result.
 */

    for (loop = 0; loop < Num_Instruments; loop++)
    {

       /*
        *  The low byte of the instrument address is the primary
        *  address. Assign the variable PAD the primary address of the
        *  instrument. The macro GetPAD, defined in NI488.H, returns
        *  the low byte of the instrument address.
        */

           PAD = GetPAD(Result[loop]);

       /*
        *  The high byte of the instrument address is the secondary
        *  address. Assign the variable SAD the primary address of the
        *  instrument. The macro GetSAD, defined in NI488.H, returns
        *  the high byte of the instrument address.
        */

           SAD = GetSAD(Result[loop]);

           if (SAD == NO_SAD)
           {
              printf("The instrument at Result[%d]: PAD = %d SAD = NONE\n",
                      loop, PAD);
           }
           else
           {
              printf("The instrument at Result[%d]: PAD = %d SAD = %d\n",
                      loop, PAD, SAD);
           }

    }      /*  End of FOR loop */


/*  Take the board offline.                                                 */

    ibonl (GPIB0, 0);

    return 0;

}


void GpibError(char *msg) {
    unsigned int Status = Ibsta();
    unsigned int Error = Iberr();
    unsigned int Count = Ibcnt();

    printf ("%s\n", msg);

    printf ("ibsta = &H%x  <", Status);
    if (Status & ERR )  printf (" ERR");
    if (Status & TIMO)  printf (" TIMO");
    if (Status & END )  printf (" END");
    if (Status & SRQI)  printf (" SRQI");
    if (Status & RQS )  printf (" RQS");
    if (Status & CMPL)  printf (" CMPL");
    if (Status & LOK )  printf (" LOK");
    if (Status & REM )  printf (" REM");
    if (Status & CIC )  printf (" CIC");
    if (Status & ATN )  printf (" ATN");
    if (Status & TACS)  printf (" TACS");
    if (Status & LACS)  printf (" LACS");
    if (Status & DTAS)  printf (" DTAS");
    if (Status & DCAS)  printf (" DCAS");
    printf (" >\n");


    printf ("iberr = %d", Error);
    if (Error == EDVR) printf (" EDVR <Driver error>\n");
    if (Error == ECIC) printf (" ECIC <Not Controller-In-Charge>\n");
    if (Error == ENOL) printf (" ENOL <No Listener>\n");
    if (Error == EADR) printf (" EADR <Address error>\n");
    if (Error == EARG) printf (" EARG <Invalid argument>\n");
    if (Error == ESAC) printf (" ESAC <Not System Controller>\n");
    if (Error == EABO) printf (" EABO <Operation aborted>\n");
    if (Error == ENEB) printf (" ENEB <No GPIB board>\n");
    if (Error == EDMA) printf (" EDMA <DMA error>\n");   
    if (Error == EOIP) printf (" EOIP <Async I/O in progress>\n");
    if (Error == ECAP) printf (" ECAP <No capability>\n");
    if (Error == EFSO) printf (" EFSO <File system error>\n");
    if (Error == EBUS) printf (" EBUS <Command error>\n");
    if (Error == ESRQ) printf (" ESRQ <SRQ stuck on>\n");
    if (Error == ETAB) printf (" ETAB <Table Overflow>\n");
    if (Error == ELCK) printf (" ELCK <Interface is locked>\n");
    if (Error == EARM) printf (" EARM <ibnotify callback failed to rearm>\n");
    if (Error == EHDL) printf (" EHDL <Input handle is invalid>\n");
    if (Error == EWIP) printf (" EWIP <Wait in progress on specified input handle>\n");
    if (Error == ERST) printf (" ERST <The event notification was cancelled due to a reset of the interface>\n");
    if (Error == EPWR) printf (" EPWR <The interface lost power>\n");
    
    printf ("ibcnt = %u\n", Count);
    printf ("\n");

    /* Call ibonl to take the interface offline */
    ibonl (GPIB0,0);

    exit(1);
}
